@@ -1,10 +1,10 @@
 {-# LANGUAGE CPP, LambdaCase, TupleSections, DeriveTraversable #-}
-{-# LANGUAGE GeneralizedNewtypeDeriving #-}
+{-# LANGUAGE GeneralizedNewtypeDeriving, DerivingVia #-}
 {-# OPTIONS_GHC -fno-warn-unused-imports #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module     : Control.Selective
--- Copyright  : (c) Andrey Mokhov 2018-2019
+-- Copyright  : (c) Andrey Mokhov 2018-2023
 -- License    : MIT (see the file LICENSE)
 -- Maintainer : andrey.mokhov@gmail.com
 -- Stability  : experimental
@@ -27,7 +27,7 @@ module Control.Selective (
     SelectA (..), SelectM (..), Over (..), Under (..), Validation (..),
 
     -- * Miscellaneous
-    swapEither, ComposeEither (..)
+    swapEither, ComposeEither (..), ComposeTraversable (..)
     ) where
 
 import Control.Applicative
@@ -35,7 +35,6 @@ import Control.Applicative.Lift
 import Control.Arrow
 import Control.Monad.ST
 import Control.Monad.Trans.Cont
-import Control.Monad.Trans.Except
 import Control.Monad.Trans.Identity
 import Control.Monad.Trans.Maybe
 import Control.Monad.Trans.Reader
@@ -197,10 +196,10 @@ branch x l r = fmap (fmap Left) x <*? fmap (fmap Right) l <*? r
 selectA :: Applicative f => f (Either a b) -> f (a -> b) -> f b
 selectA x y = (\e f -> either f id e) <$> x <*> y
 
--- | If a functor is both 'Applicative' and 'Traversable', we can implement
--- 'select' in another interesting way: the effects associated with the second
--- argument can be skipped as long as the first argument contains only 'Right's.
-selectT :: (Applicative f, Traversable f) => f (Either a b) -> f (a -> b) -> f b
+-- | For traversable functors, we can implement 'select' in another interesting
+-- way: the effects associated with the second argument can be skipped as long
+-- as the first argument contains only 'Right' values.
+selectT :: Traversable f => f (Either a b) -> f (a -> b) -> f b
 selectT x y = case sequenceA x of
     Left  a  -> ($a) <$> y
     Right fb -> fb
@@ -275,7 +274,7 @@ matchS (Cases cs _) x f = foldr (\c -> eliminate c (f c)) (Left <$> x) cs
 matchM :: Monad m => Cases a -> m a -> (a -> m b) -> m (Either a b)
 matchM (Cases _ p) mx f = do
     x <- mx
-    if p x then Right <$> f x else return (Left x)
+    if p x then Right <$> f x else pure (Left x)
 
 -- TODO: Add a type-safe version based on @KnownNat@.
 -- | A restricted version of monadic bind. Fails with an error if the 'Bounded'
@@ -302,7 +301,14 @@ fromMaybeS x mx = select (maybe (Left ()) Right <$> mx) (const <$> x)
 
 -- | Return the first @Right@ value. If both are @Left@'s, accumulate errors.
 orElse :: (Selective f, Semigroup e) => f (Either e a) -> f (Either e a) -> f (Either e a)
-orElse x y = branch x (flip appendLeft <$> y) (pure Right)
+orElse x y = select (prepare <$> x) (combine <$> y)
+  where
+    prepare :: Either e a -> Either e (Either e a)
+    prepare = fmap Right
+
+    combine :: Semigroup e => Either e a -> e -> Either e a
+    combine (Left ey) ex = Left (ex <> ey)
+    combine (Right a) _  = Right a
 
 -- | Accumulate the @Right@ values, or return the first @Left@.
 andAlso :: (Selective f, Semigroup a) => f (Either e a) -> f (Either e a) -> f (Either e a)
@@ -312,11 +318,6 @@ andAlso x y = swapEither <$> orElse (swapEither <$> x) (swapEither <$> y)
 swapEither :: Either a b -> Either b a
 swapEither = either Right Left
 
--- | Append two semigroup values or return the @Right@ one.
-appendLeft :: Semigroup a => a -> Either a b -> Either a b
-appendLeft a1 (Left a2) = Left (a1 <> a2)
-appendLeft _  (Right b) = Right b
-
 -- | Keep checking an effectful condition while it holds.
 whileS :: Selective f => f Bool -> f ()
 whileS act = whenS act (whileS act)
@@ -348,11 +349,7 @@ allS :: Selective f => (a -> f Bool) -> [a] -> f Bool
 allS p = foldr ((<&&>) . p) (pure True)
 
 -- | Generalised folding with the short-circuiting behaviour.
-foldS :: (Selective f, Foldable t, Monoid a
-#if !MIN_VERSION_base(4,11,0)
-  , Semigroup a
-#endif
-    ) => t (f (Either e a)) -> f (Either e a)
+foldS :: (Selective f, Foldable t, Monoid a) => t (f (Either e a)) -> f (Either e a)
 foldS = foldr andAlso (pure (Right mempty))
 
 -- Instances
@@ -374,8 +371,8 @@ instance Applicative f => Selective (SelectA f) where
 
 -- Note: Validation e a ~ Lift (Under e) a
 instance Selective f => Selective (Lift f) where
+    select (Pure (Right x)) _         = Pure x -- Lazy in the second argument
     select              x   (Pure  y) = either y id <$> x
-    select (Pure (Right x)) _         = Pure x
     select (Pure (Left  x)) (Other y) = Other $ ($x) <$> y
     select (Other       x ) (Other y) = Other $   x  <*? y
 
@@ -397,10 +394,7 @@ instance Monad f => Selective (SelectM f) where
 -- | Static analysis of selective functors with over-approximation.
 newtype Over m a = Over { getOver :: m }
     deriving (Eq, Functor, Ord, Show)
-
-instance Monoid m => Applicative (Over m) where
-    pure _            = Over mempty
-    Over x <*> Over y = Over (mappend x y)
+    deriving Applicative via (Const m)
 
 -- select = selectA
 instance Monoid m => Selective (Over m) where
@@ -409,10 +403,7 @@ instance Monoid m => Selective (Over m) where
 -- | Static analysis of selective functors with under-approximation.
 newtype Under m a = Under { getUnder :: m }
     deriving (Eq, Functor, Ord, Show, Foldable, Traversable)
-
-instance Monoid m => Applicative (Under m) where
-    pure _              = Under mempty
-    Under x <*> Under y = Under (mappend x y)
+    deriving Applicative via (Const m)
 
 -- select = selectT
 instance Monoid m => Selective (Under m) where
@@ -510,7 +501,6 @@ instance             Selective (ST s)     where select = selectM
 instance             Selective STM        where select = selectM
 
 instance                        Selective (ContT      r m) where select = selectM
-instance            Monad m  => Selective (ExceptT    e m) where select = selectM
 instance            Monad m  => Selective (MaybeT       m) where select = selectM
 instance (Monoid w, Monad m) => Selective (RWST   r w s m) where select = selectM
 instance (Monoid w, Monad m) => Selective (S.RWST r w s m) where select = selectM
@@ -529,20 +519,42 @@ instance ArrowChoice a => Selective (ArrowMonad a) where
 toArrow :: Arrow a => ArrowMonad a (i -> o) -> a i o
 toArrow (ArrowMonad f) = arr ((),) >>> first f >>> arr (uncurry ($))
 
----------------------------------- Alternative ---------------------------------
--- | Composition of a functor @f@ with the 'Either' monad.
+------------------------------ ComposeTraversable ------------------------------
+-- | Composition of a selective functor @f@ and an applicative traversable
+-- functor @g@.
+newtype ComposeTraversable f g a = ComposeTraversable (f (g a))
+    deriving (Functor, Applicative) via Compose f g
+
+instance (Selective f, Applicative g, Traversable g) => Selective (ComposeTraversable f g) where
+    select (ComposeTraversable x) (ComposeTraversable f) = ComposeTraversable $
+        select (prepare <$> x) (combine <$> f)
+      where
+        prepare :: Traversable g => g (Either a b) -> Either a (g b)
+        prepare = sequenceA
+
+        combine :: Traversable g => g (a -> b) -> a -> g b
+        combine = sequenceA
+
+--------------------------------- ComposeEither --------------------------------
+-- | Composition of a selective functor @f@ with the 'Either' monad.
 newtype ComposeEither f e a = ComposeEither (f (Either e a))
-    deriving Functor
+    deriving Functor via Compose f (Either e)
+    deriving Selective via ComposeTraversable f (Either e)
 
-instance Applicative f => Applicative (ComposeEither f e) where
-    pure a                              = ComposeEither (pure $ Right a)
-    ComposeEither x <*> ComposeEither y = ComposeEither ((<*>) <$> x <*> y)
+instance Selective f => Applicative (ComposeEither f e) where
+    pure = ComposeEither . pure . Right
 
-instance (Selective f, Monoid e
-#if !MIN_VERSION_base(4,11,0)
-  , Semigroup e
-#endif
-    ) => Alternative (ComposeEither f e) where
+    ComposeEither f <*> ComposeEither a = ComposeEither $
+        select (prepare <$> f) (combine <$> a)
+      where
+        prepare :: Either e (a -> b) -> Either (a -> b) (Either e b)
+        prepare = either (Right . Left) Left
+
+        combine :: Either e a -> (a -> b) -> Either e b
+        combine = flip fmap
+
+---------------------------------- Alternative ---------------------------------
+instance (Selective f, Monoid e) => Alternative (ComposeEither f e) where
     empty                               = ComposeEither (pure $ Left mempty)
     ComposeEither x <|> ComposeEither y = ComposeEither (x `orElse` y)
 
@@ -559,7 +571,7 @@ This has two issues:
 1) A generic 'failIfLeft' if not possible, although many actual instances should
    be able to implement it.
 
-2) More importantly, this requires duplication of work: if we failed becauase we
+2) More importantly, this requires duplication of work: if we failed because we
    happened to parse a 'Left' value in the first parser, then we need to rerun
    it, obtain a 'Left' once again, and then execute the second parser. Again, a
    specific instance may be able to cache the result and reuse it without
